import java.util.*;
import java.io.*;
import java.sql.*;


/*----------------------------------------------------------------- Entry class
|																			  |
|				 Wrapper class for an Entry in the GGLE db					  |
|																			  |
|																			  |
|																			  |
| 																			  |
\----------------------------------------------------------------------------*/
public class Entry
{
	private int 			entryId;
	private String 			domain	= "";
	private String 			path	= "";
	private String 			page	= "";
	private String 			url		= "";
	private java.sql.Date 	doc;
	private java.sql.Date 	dom;
	private int 			size;
	private Vector 			links;


	/*------------------------------------------------------------- constructor
	|																		  |
	| standart constructor													  |
	\------------------------------------------------------------------------*/
	public Entry(String u, long dc, long dm, int s, Vector l)
	{
		int first = u.indexOf('/', 8);
		int last = u.lastIndexOf('/') + 1;

		if( first != -1 )
		{
			domain = u.substring( 0, first );
			path = u.substring( first, last );
			page = u.substring( last );
		}
		else
		{
			domain = u;
		}
		url  = u;

		doc = new java.sql.Date(dc);
		dom = new java.sql.Date(dm);
		size = s;
		links = l;
		setEntryId(0);

	}

	/*------------------------------------------------------------ setEntryId()
	|																		  |
	| 																		  |
	\------------------------------------------------------------------------*/
	public void setEntryId( int id )
	{
		entryId = id;

		if( links != null )
		{
			for( int i = 0; i < links.size(); i++ )
			{
				((Link) links.elementAt(i)).setEntry(id);
			}
		}
	}


	/*--------------------------------------------------------- get() functions
	|																		  |
	| 																		  |
	\------------------------------------------------------------------------*/
	public int getEntryId()
	{
		return entryId;
	}

	public String getUrl()
	{
		return url ;
	}

	public String getDomain()
	{
		return domain ;
	}

	public String getPath()
	{
		return url ;
	}

	public String getPage()
	{
		return page ;
	}

	public java.sql.Date getDoc()
	{
		return doc;
	}

	public java.sql.Date getDom()
	{
		return dom;
	}

	public int getSize()
	{
		return size;
	}

	public Vector getLinks()
	{
		return links;
	}



	/*------------------------------------------------------------ printEntry()
	|																		  |
	| 																		  |
	\------------------------------------------------------------------------*/
	public void printEntry()
	{
		System.out.println("entryId:\t" 	+ entryId);
		System.out.println("domain:\t\t" 	+ domain);
		System.out.println("path:\t\t" 		+ path);
		System.out.println("page:\t\t" 		+ page);
		System.out.println("url:\t\t" 		+ url);
		System.out.println("dom:\t\t" 		+ dom);
		System.out.println("doc:\t\t" 		+ doc);
		System.out.println("size:\t\t"	 	+ size);

		System.out.println("LINKS:");

		Link link;

		for (Enumeration l = links.elements() ; l.hasMoreElements();)
		{
			//  --------- split the link u into domain path and page
			link = (Link) l.nextElement();
			System.out.println("\turl:\t\t" 	+ link.getUrl() );
			System.out.println("\tdomain:\t\t" 	+ link.getDomain() );
			System.out.println("\tpath:\t\t" 	+ link.getPath() );
			System.out.println("\tpage:\t\t" 	+ link.getPage() + "\n");

		}

		System.out.println("\n");
	}

	/*-------------------------------------------------------------- addBatch()
	|																		  |
	| 																		  |
	\------------------------------------------------------------------------*/
	public int addBatch( PreparedStatement entryPstmt,
						 PreparedStatement linkPstmt,
						int 			   linkId	 )
	{
		try
		{
			entryPstmt.setInt		(	1, entryId			);
			entryPstmt.setString	(	2, domain  			);
			entryPstmt.setString	(	3, path				);
			entryPstmt.setString	(	4, page				);
			entryPstmt.setString	(	5, url				);
			entryPstmt.setDate		(	6, doc				);
			entryPstmt.setDate		(	7, dom				);
			entryPstmt.setInt		(	8, size				);

			entryPstmt.addBatch();
		}
		catch( Exception e )
		{
			System.out.println(
			"ERROR while preparing insert statment for entry '" + url
			+ "'\nEXCEPTION: " + e);
		}

		Link link;

		for (Enumeration lnks = links.elements() ; lnks.hasMoreElements();)
		{
			link = (Link) lnks.nextElement();
			link.setLinkId(linkId);

			link.addBatch(linkPstmt);
			linkId++;
		}



		return linkId;
	}
}