import java.util.*;
import java.io.*;
import java.sql.*;


/*-----------------------------------------------------ParallelConnection class
|																			  |
|				 Thread to handle SQL Statments in parallel					  |
|																			  |
|																			  |
|																			  |
| 																			  |
\----------------------------------------------------------------------------*/
public class ParallelConnection extends Thread
{
	private Connection con					= null;
	private Hashtable stmts					= null;
	private Hashtable pstmts				= null;
	private Hashtable originalSql			= null;
	private Hashtable results				= null;
	private Statement stmt, sTmp			= null;
	private PreparedStatement pstmt, pTmp	= null;
	private Object stmtKey					= null;
	private String sql						= null;
	private boolean working, locked			= false;
	private int execute 					= -1;

	/*------------------------------------------------------------- constructor
	|																		  |
	| empty constructor														  |
	\------------------------------------------------------------------------*/
	public ParallelConnection( )
	{
		stmts	 	= new Hashtable();
		pstmts	 	= new Hashtable();
		originalSql = new Hashtable();
		results 	= new Hashtable();
	}

	/*------------------------------------------------------------- constructor
	|																		  |
	| standart constructor													  |
	\------------------------------------------------------------------------*/
	public ParallelConnection( Connection c )
	{
		con      	= c;
		stmts	 	= new Hashtable();
		pstmts	 	= new Hashtable();
		originalSql = new Hashtable();
		results 	= new Hashtable();
	}


	/*------------------------------------------------------- createStatement()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void createStatement( Object stmtK )
		throws SQLException
	{
		stmts.put( stmtK, con.createStatement() );
	}

	/*------------------------------------------------------ prepareStatement()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void prepareStatement( Object stmtK, String sql )
		throws SQLException
	{
		pstmts.put( stmtK, con.prepareStatement(sql) );
		originalSql.put( stmtK, sql );
	}

	/*-------------------------------------------------------------- addBatch()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void addBatch( Object stmtK )
		throws SQLException
	{
		while( locked )
		{
			try { wait(); }
			catch(InterruptedException e) {	System.out.println(e);  }
		}

		pTmp = (PreparedStatement) pstmts.get( stmtK );

		if( pTmp != null )
		{
			pTmp.addBatch();
		}
		else
		{
			System.out.println(
				"ERROR: PreparedStatement '" + stmtK + "' doesn't exist!");
		}

		pTmp = null;
	}

	/*---------------------------------------------------- addBatch(String sql)
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void addBatch( Object stmtK, String sql )
		throws SQLException
	{
		while( locked )
		{
			try { wait(); }
			catch(InterruptedException e) {	System.out.println(e);  }
		}

		sTmp = (Statement) stmts.get( stmtK );

		if( sTmp != null )
		{
			sTmp.addBatch( sql );
		}
		else
		{
			System.out.println(
				"ERROR: Statement '" + stmtK + "' doesn't exist!");
		}

		sTmp = null;
	}


	/*---------------------------------------------------------------- setInt()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void setInt( Object stmtK, int num, int i )
		throws SQLException
	{

		while( locked )
		{
			try { wait(); }
			catch(InterruptedException e) {	System.out.println(e);  }
		}

		pTmp = (PreparedStatement) pstmts.get( stmtK );

		if( pTmp != null )
		{
			pTmp.setInt(num, i);
		}
		else
		{
			System.out.println(
				"ERROR: PreparedStatement '" + stmtK + "' doesn't exist!");
		}

		pTmp = null;
	}

	/*--------------------------------------------------------------- setDate()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void setDate( Object stmtK, int num, java.sql.Date d )
			throws SQLException
	{
		while( locked )
		{
			try { wait(); }
			catch(InterruptedException e) {	System.out.println(e);  }
		}


		pTmp = (PreparedStatement) pstmts.get( stmtK );

		if( pTmp != null )
		{
			pTmp.setDate(num, d);
		}
		else
		{
			System.out.println(
				"ERROR: PreparedStatement '" + stmtK + "' doesn't exist!");
		}

		pTmp = null;
	}

	/*------------------------------------------------------------- setString()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void setString( Object stmtK, int num, String s )
			throws SQLException
	{

		while( locked )
		{
			try { wait(); }
			catch(InterruptedException e) {	System.out.println(e);  }
		}

		pTmp = (PreparedStatement) pstmts.get( stmtK );

		if( pTmp != null )
		{
			pTmp.setString(num, s);
		}
		else
		{
			System.out.println(
				"ERROR: PreparedStatement '" + stmtK + "' doesn't exist!");
		}

		pTmp = null;
	}


	/*---------------------------------------------------------- getStatement()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized Statement getStatement( Object stmtK )
	{
		//----------------------- can't retrive statment while locked
		while( locked )
		{
			try	{ wait();}
			catch(InterruptedException e) { System.out.println(e); }
		}


		return ( (Statement) stmts.get( stmtK ));
	}

	/*--------------------------------------------------------- getPStatement()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized PreparedStatement getPStatement( Object stmtK )
	{
		//----------------------- can't retrive statment while locked
		while( locked )
		{
			try	{ wait();}
			catch(InterruptedException e) { System.out.println(e); }
		}


		return ( (PreparedStatement) pstmts.get( stmtK ));
	}


	/*------------------------------------------------------------- getResult()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized ResultSet getResult( Object stmtK )
	{

		//----------- can't retrive resultSet while working or locked
		while( working || locked )
		{
			try	{ wait();}
			catch(InterruptedException e) { System.out.println(e); }
		}

		//---------------------------------- remove and return result
		return ( (ResultSet) results.remove( stmtK ));
	}


	/*---------------------------------------------------------- executeQuery()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void executeQuery( Object stmtK )
	{

		// --------- if this thread is allready executing a stmt wait
		while( working || locked )
		{
			try { wait(); }
			catch(InterruptedException e) {	System.out.println(e);  }
		}

		if( results.containsKey(stmtK) )
			System.out.println(
				"WARNING: An unretrieved result set exists for " + stmtK
			+	". The result set will get overwritten");

		// ---------------------- tell this thread to start executing
		execute = 1;
		stmtKey = stmtK;

		// -------------------- lock statement till a new copy exists
		locked = true;

		notifyAll();
	}

	/*--------------------------------------------------- executeQuery(String s)
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void executeQuery( Object stmtK, String s )
	{

		// --------- if this thread is allready executing a stmt wait
		while( working || locked )
		{
			try { wait(); }
			catch(InterruptedException e) {	System.out.println(e);  }
		}

		if( results.containsKey(stmtK) )
			System.out.println(
				"WARNING: An unretrieved result set exists for " + stmtK
			+	". The result set will get overwritten");

		// ---------------------- tell this thread to start executing
		execute = 1;
		stmtKey = stmtK;
		sql		= s;

		// -------------------- lock statement till a new copy exists
		locked = true;

		notifyAll();
	}




	/*--------------------------------------------------------- executeUpdate()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void executeUpdate( Object stmtK )
	{

		// --------- if this thread is allready executing a stmt wait
		while( working || locked )
		{
			try { wait(); }
			catch(InterruptedException e) {	System.out.println(e);  }
		}

		// ---------------------- tell this thread to start executing
		execute = 2;
		stmtKey = stmtK;

		// -------------------- lock statement till a new copy exists
		locked = true;

		notifyAll();
	}

	/*------------------------------------------------- executeUpdate(String s)
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void executeUpdate( Object stmtK, String s )
	{

		// --------- if this thread is allready executing a stmt wait
		while( working || locked )
		{
			try { wait(); }
			catch(InterruptedException e) {	System.out.println(e);  }
		}

		// ---------------------- tell this thread to start executing
		execute = 2;
		stmtKey = stmtK;
		sql		= s;

		// -------------------- lock statement till a new copy exists
		locked = true;

		notifyAll();
	}

	/*---------------------------------------------------------- executeBatch()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void executeBatch( Object stmtK )
	{

		// --------- if this thread is allready executing a stmt wait
		while( working || locked )
		{
			try { wait(); }
			catch(InterruptedException e) {	System.out.println(e);  }
		}

		// ---------------------- tell this thread to start executing
		execute = 3;
		stmtKey = stmtK;

		// -------------------- lock statement till a new copy exists
		locked = true;

		notifyAll();
	}


	/*-------------------------------------------------- executeQueryInternal()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	private void executeQueryInternal( Object stmtK )
	{
		try
		{
			// ------------ adds new copy of requested statement to stmts
			newStatement( stmtK );

			// ----------------------------------------------- run update
			// --------- check if requested stmt is a stmt
			if( stmt != null )
			{
				results.put( stmtK, stmt.executeQuery(sql) );
				//stmt.close();
				stmt = null;
				sql  = null;
			}
			// --------- check if requested stmt is a pstmt
			else if( pstmt != null )
			{
				results.put( stmtK, pstmt.executeQuery() );
				//pstmt.close();
				pstmt = null;
			}
		}
		catch( SQLException e )
		{
			System.out.println(e);
			try{  con.rollback(); } catch(Exception ex) {};
		}

		execute = -1;
	}



	/*------------------------------------------------- executeUpdateInternal()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	private void executeUpdateInternal( Object stmtK )
	{
		try
		{
			// ------------ adds new copy of requested statement to stmts
			newStatement( stmtK );

			// ----------------------------------------------- run update
			// --------- check if requested stmt is a stmt
			if( stmt != null )
			{
				stmt.executeUpdate(sql);
				stmt.close();
				stmt = null;
				sql  = null;
			}
			// --------- check if requested stmt is a pstmt
			else if( pstmt != null )
			{
				pstmt.executeUpdate();
				pstmt.close();
				pstmt = null;
			}
		}
		catch( SQLException e )
		{
			System.out.println(e);
			try{  con.rollback(); } catch(Exception ex){};
		}

		execute = -1;
	}


	/*-------------------------------------------------- executeBatchInternal()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	private void executeBatchInternal( Object stmtK )
	{
		try
		{
			// ------------ adds new copy of requested statement to stmts
			newStatement( stmtK );

			// ----------------------------------------------- run update
			// --------- check if requested stmt is a stmt
			if( stmt != null )
			{
				stmt.executeBatch();
				stmt.close();
				stmt = null;
			}
			// --------- check if requested stmt is a pstmt
			else if( pstmt != null )
			{
				pstmt.executeBatch();
				pstmt.close();
				pstmt = null;
			}
		}
		catch( Exception e )
		{
			System.out.println(e);
			try{  con.rollback(); } catch(Exception ex){};
		}

		execute = -1;
	}

	/*---------------------------------------------------------- newStatement()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void newStatement( Object stmtK ) throws SQLException
	{

		if( stmts.containsKey(stmtK)  )
		{
			stmt = (Statement) stmts.remove(stmtKey);

			stmts.put( stmtKey, con.createStatement() );
			pstmt = null;
		}
		else if( pstmts.containsKey(stmtK) )
		{
			pstmt = (PreparedStatement) pstmts.remove(stmtK);

			pstmts.put
			(
				stmtK,
				con.prepareStatement( (	String) originalSql.get(stmtK) )
			);
			stmt = null;
		}
		else
		{
			System.out.println(
				"ERROR: Statement '" + stmtK + "' doesn't exist!");
		}

		locked = false;
		notifyAll();
	}


	/*------------------------------------------------------------------ halt()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void halt()
	{
		while( working || locked )
		{
			try	{ wait();}
			catch(InterruptedException e) { System.out.println(e); }
		}

		execute = 0;

		//System.out.println("THREAD: halt called");
		notifyAll();
	}


	/*---------------------------------------------------------------- getJob()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized int getJob( )
	{
		while( execute < 0 )
		{
			try { wait(); }
			catch(InterruptedException e) {	System.out.println(e);  }
		}

		working = true;

		return execute;
	}

	/*--------------------------------------------------------setWorkFinished()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void setWorkFinished( )
	{
		working = false;
		notifyAll();
	}

	/*--------------------------------------------------------setWorkFinished()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public synchronized void cleanUp( )
	{
		// --------------------------------------- close all sql statments
		Enumeration st;

		try
		{

			for (  st = stmts.elements() ; st.hasMoreElements(); )
			{
				( (Statement) st.nextElement() ).close();
			}
			for ( st = pstmts.elements() ; st.hasMoreElements();)
			{
				( (PreparedStatement) st.nextElement() ).close();
			}
		}
		catch( SQLException e )
		{
			System.out.println( e );
		}
	}



	/*------------------------------------------------------------------- run()
	|																		  |
	| DOCUMENT ME!															  |
	\------------------------------------------------------------------------*/
	public void run( )
	{
		SQLWarning warn;
		int job = -1;


		while( job != 0 )
		{
			job = getJob();

			switch(job)
			{

				case 0:
					cleanUp();
					break;

				case 1:
					executeQueryInternal( stmtKey );
					break;

				case 2:
					executeUpdateInternal( stmtKey );
					break;

				case 3:
					executeBatchInternal( stmtKey );
					break;

				default:
					System.out.println(
						"FATAL ERROR: This should never happen!");
			}

			setWorkFinished();
		}
	}
}